@@ -350,8 +350,9 @@ do_io_tests (void)
 {
 	GInputStream *imem, *islow, *in;
 	GOutputStream *omem, *oslow, *out;
-	char *raw_contents, *buf;
-	gsize raw_length;
+	GMemoryOutputStream *mem;
+	SoupBuffer *raw_contents;
+	char *buf;
 	GString *chunkified;
 	GError *error = NULL;
 	gssize nread, nwrote, total;
@@ -359,13 +360,8 @@ do_io_tests (void)
 
 	debug_printf (1, "\nI/O tests\n");
 
-	if (!g_file_get_contents (SRCDIR "/index.txt", &raw_contents, &raw_length, &error)) {
-		g_printerr ("Could not read index.txt: %s\n",
-			    error->message);
-		exit (1);
-	}
-
-	chunkified = chunkify (raw_contents, raw_length);
+	raw_contents = soup_test_get_index ();
+	chunkified = chunkify (raw_contents->data, raw_contents->length);
 
 	debug_printf (1, "  sync read\n");
 
@@ -382,40 +378,26 @@ do_io_tests (void)
 	g_object_unref (imem);
 	g_object_unref (islow);
 
-	buf = g_malloc (raw_length);
+	buf = g_malloc (raw_contents->length);
 	total = 0;
 	while (TRUE) {
-		nread = g_input_stream_read (in, buf + total, raw_length - total,
+		nread = g_input_stream_read (in, buf + total,
+					     raw_contents->length - total,
 					     NULL, &error);
-		if (nread == -1) {
-			debug_printf (1, "  Error reading stream: %s\n", error->message);
-			g_clear_error (&error);
-			errors++;
-			break;
-		} else if (nread == 0)
-			break;
-		else
+		g_assert_no_error (error);
+		g_clear_error (&error);
+		if (nread > 0)
 			total += nread;
+		else
+			break;
 	}
 
 	g_input_stream_close (in, NULL, &error);
-	if (error) {
-		debug_printf (1, "  Error closing input stream: %s\n", error->message);
-		g_clear_error (&error);
-		errors++;
-	}
+	g_assert_no_error (error);
+	g_clear_error (&error);
 	g_object_unref (in);
 
-	if (total == raw_length) {
-		if (memcmp (buf, raw_contents, raw_length) != 0) {
-			debug_printf (1, "  mismatch when reading\n");
-			errors++;
-		}
-	} else {
-		debug_printf (1, "  incorrect read length: %d vs %d\n",
-			      (int) total, (int) raw_length);
-		errors++;
-	}
+	soup_assert_cmpmem (buf, total, raw_contents->data, raw_contents->length);
 	g_free (buf);
 
 	debug_printf (1, "  async read\n");
@@ -433,11 +415,12 @@ do_io_tests (void)
 	g_object_unref (imem);
 	g_object_unref (islow);
 
-	buf = g_malloc (raw_length);
+	buf = g_malloc (raw_contents->length);
 	total = 0;
 	while (TRUE) {
 		nread = g_pollable_input_stream_read_nonblocking (G_POLLABLE_INPUT_STREAM (in),
-								  buf + total, raw_length - total,
+								  buf + total,
+								  raw_contents->length - total,
 								  NULL, &error);
 		if (nread == -1 && g_error_matches (error, G_IO_ERROR, G_IO_ERROR_WOULD_BLOCK)) {
 			GSource *source;
@@ -452,9 +435,8 @@ do_io_tests (void)
 			g_source_unref (source);
 			continue;
 		} else if (nread == -1) {
-			debug_printf (1, "  Error reading stream: %s\n", error->message);
+			g_assert_no_error (error);
 			g_clear_error (&error);
-			errors++;
 			break;
 		} else if (nread == 0)
 			break;
@@ -463,23 +445,11 @@ do_io_tests (void)
 	}
 
 	g_input_stream_close (in, NULL, &error);
-	if (error) {
-		debug_printf (1, "  Error closing input stream: %s\n", error->message);
-		g_clear_error (&error);
-		errors++;
-	}
+	g_assert_no_error (error);
+	g_clear_error (&error);
 	g_object_unref (in);
 
-	if (total == raw_length) {
-		if (memcmp (buf, raw_contents, raw_length) != 0) {
-			debug_printf (1, "  mismatch when reading\n");
-			errors++;
-		}
-	} else {
-		debug_printf (1, "  incorrect read length: %d vs %d\n",
-			      (int) total, (int) raw_length);
-		errors++;
-	}
+	soup_assert_cmpmem (buf, total, raw_contents->data, raw_contents->length);
 	g_free (buf);
 
 	debug_printf (1, "  sync write\n");
@@ -499,39 +469,32 @@ do_io_tests (void)
 	g_object_unref (oslow);
 
 	total = chunk_length = chunk_total = 0;
-	while (total < raw_length) {
+	while (total < raw_contents->length) {
 		if (chunk_total == chunk_length) {
-			chunk_length = MIN (CHUNK_SIZE, raw_length - total);
+			chunk_length = MIN (CHUNK_SIZE, raw_contents->length - total);
 			chunk_total = 0;
 		}
-		nwrote = g_output_stream_write (out, raw_contents + total,
+		nwrote = g_output_stream_write (out, raw_contents->data + total,
 						chunk_length - chunk_total, NULL, &error);
-		if (nwrote == -1) {
-			debug_printf (1, "  Error writing stream: %s\n", error->message);
-			g_clear_error (&error);
-			errors++;
-			break;
-		} else {
+		g_assert_no_error (error);
+		g_clear_error (&error);
+		if (nwrote > 0) {
 			total += nwrote;
 			chunk_total += nwrote;
-		}
+		} else
+			break;
 	}
 
 	g_output_stream_close (out, NULL, &error);
-	if (error) {
-		debug_printf (1, "  Error closing output stream: %s\n", error->message);
-		g_clear_error (&error);
-		errors++;
-	}
-	g_object_unref (out);
+	g_assert_no_error (error);
+	g_clear_error (&error);
 
-	if (total == raw_length) {
-		if (memcmp (buf, chunkified->str, chunkified->len) != 0) {
-			debug_printf (1, "  mismatch when writing\n");
-			g_print ("%.*s\n", (int)chunkified->len, buf);
-			errors++;
-		}
-	}
+	mem = G_MEMORY_OUTPUT_STREAM (omem);
+	soup_assert_cmpmem (g_memory_output_stream_get_data (mem),
+			    g_memory_output_stream_get_data_size (mem),
+			    chunkified->str, chunkified->len);
+
+	g_object_unref (out);
 	g_free (buf);
 
 	debug_printf (1, "  async write\n");
@@ -551,13 +514,13 @@ do_io_tests (void)
 	g_object_unref (oslow);
 
 	total = chunk_length = chunk_total = 0;
-	while (total < raw_length) {
+	while (total < raw_contents->length) {
 		if (chunk_total == chunk_length) {
-			chunk_length = MIN (CHUNK_SIZE, raw_length - total);
+			chunk_length = MIN (CHUNK_SIZE, raw_contents->length - total);
 			chunk_total = 0;
 		}
 		nwrote = g_pollable_output_stream_write_nonblocking (G_POLLABLE_OUTPUT_STREAM (out),
-								     raw_contents + total,
+								     raw_contents->data + total,
 								     chunk_length - chunk_total,
 								     NULL, &error);
 		if (nwrote == -1 && g_error_matches (error, G_IO_ERROR, G_IO_ERROR_WOULD_BLOCK)) {
@@ -573,9 +536,8 @@ do_io_tests (void)
 			g_source_unref (source);
 			continue;
 		} else if (nwrote == -1) {
-			debug_printf (1, "  Error writing stream: %s\n", error->message);
+			g_assert_no_error (error);
 			g_clear_error (&error);
-			errors++;
 			break;
 		} else {
 			total += nwrote;
@@ -584,19 +546,15 @@ do_io_tests (void)
 	}
 
 	g_output_stream_close (out, NULL, &error);
-	if (error) {
-		debug_printf (1, "  Error closing output stream: %s\n", error->message);
-		g_clear_error (&error);
-		errors++;
-	}
-	g_object_unref (out);
+	g_assert_no_error (error);
+	g_clear_error (&error);
 
-	if (total == raw_length) {
-		if (memcmp (buf, chunkified->str, chunkified->len) != 0) {
-			debug_printf (1, "  mismatch when writing\n");
-			errors++;
-		}
-	}
+	mem = G_MEMORY_OUTPUT_STREAM (omem);
+	soup_assert_cmpmem (g_memory_output_stream_get_data (mem),
+			    g_memory_output_stream_get_data_size (mem),
+			    chunkified->str, chunkified->len);
+
+	g_object_unref (out);
 	g_free (buf);
 
 	debug_printf (1, "  failed write\n");
@@ -617,19 +575,16 @@ do_io_tests (void)
 	g_object_unref (oslow);
 
 	total = 0;
-	while (total < raw_length) {
-		nwrote = g_output_stream_write (out, raw_contents + total,
-						raw_length - total, NULL, NULL);
+	while (total < raw_contents->length) {
+		nwrote = g_output_stream_write (out, raw_contents->data + total,
+						raw_contents->length - total, NULL, NULL);
 		if (nwrote == -1)
 			break;
 		else
 			total += nwrote;
 	}
 
-	if (total == raw_length) {
-		debug_printf (1, "  breaking stream didn't break?\n");
-		errors++;
-	}
+	g_assert_cmpint (total, !=, raw_contents->length);
 
 	g_output_stream_close (out, NULL, NULL);
 	g_object_unref (out);
@@ -637,18 +592,21 @@ do_io_tests (void)
 	g_free (buf);
 
 	g_string_free (chunkified, TRUE);
-	g_free (raw_contents);
 }
 
 int
 main (int argc, char **argv)
 {
+	int ret;
+
 	test_init (argc, argv, NULL);
 
 	force_io_streams_init ();
 
-	do_io_tests ();
+	g_test_add_func ("/chunk-io", do_io_tests);
+
+	ret = g_test_run ();
 
 	test_cleanup ();
-	return errors != 0;
+	return ret;
 }
