@@ -12,9 +12,9 @@ static gboolean apache_running;
 #endif
 
 static SoupLogger *logger;
+static SoupBuffer *index_buffer;
 
-int debug_level, errors;
-gboolean parallelize = TRUE;
+int debug_level;
 gboolean expect_warning, tls_available;
 static int http_debug_level;
 
@@ -38,9 +38,6 @@ static GOptionEntry debug_entry[] = {
 	{ "debug", 'd', G_OPTION_FLAG_NO_ARG,
 	  G_OPTION_ARG_CALLBACK, increment_debug_level,
 	  "Enable (or increase) test-specific debugging", NULL },
-	{ "parallel", 'p', G_OPTION_FLAG_REVERSE,
-	  G_OPTION_ARG_NONE, &parallelize,
-	  "Toggle parallelization (default is on, unless -d or -h)", NULL },
 	{ "http-debug", 'h', G_OPTION_FLAG_NO_ARG,
 	  G_OPTION_ARG_CALLBACK, increment_http_debug_level,
 	  "Enable (or increase) HTTP-level debugging", NULL },
@@ -58,21 +55,6 @@ quit (int sig)
 	exit (1);
 }
 
-static void
-test_log_handler (const char *log_domain, GLogLevelFlags log_level,
-		  const char *message, gpointer user_data)
-{
-	if (log_level & (G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL)) {
-		if (expect_warning) {
-			expect_warning = FALSE;
-			debug_printf (2, "Got expected warning: %s\n", message);
-			return;
-		} else
-			errors++;
-	}
-	g_log_default_handler (log_domain, log_level, message, user_data);
-}
-
 void
 test_init (int argc, char **argv, GOptionEntry *entries)
 {
@@ -83,6 +65,7 @@ test_init (int argc, char **argv, GOptionEntry *entries)
 
 	setlocale (LC_ALL, "");
 	g_setenv ("GSETTINGS_BACKEND", "memory", TRUE);
+	g_setenv ("GIO_USE_PROXY_RESOLVER", "dummy", TRUE);
 
 	name = strrchr (argv[0], '/');
 	if (!name++)
@@ -91,6 +74,9 @@ test_init (int argc, char **argv, GOptionEntry *entries)
 		name += 3;
 	g_set_prgname (name);
 
+	g_test_init (&argc, &argv, NULL);
+	g_test_set_nonfatal_assertions ();
+
 	opts = g_option_context_new (NULL);
 	g_option_context_add_main_entries (opts, debug_entry, NULL);
 	if (entries)
@@ -105,14 +91,9 @@ test_init (int argc, char **argv, GOptionEntry *entries)
 	}
 	g_option_context_free (opts);
 
-	if (debug_level > 0 || http_debug_level > 0)
-		parallelize = !parallelize;
-
 	/* Exit cleanly on ^C in case we're valgrinding. */
 	signal (SIGINT, quit);
 
-	g_log_set_default_handler (test_log_handler, NULL);
-
 	tls_backend = g_tls_backend_get_default ();
 	tls_available = g_tls_backend_supports_tls (tls_backend);
 }
@@ -127,16 +108,12 @@ test_cleanup (void)
 
 	if (logger)
 		g_object_unref (logger);
+	if (index_buffer)
+		soup_buffer_free (index_buffer);
 
 	g_main_context_unref (g_main_context_default ());
 
 	debug_printf (1, "\n");
-	if (errors) {
-		g_print ("%s: %d error(s).%s\n",
-			 g_get_prgname (), errors,
-			 debug_level == 0 ? " Run with '-d' for details" : "");
-	} else
-		g_print ("%s: OK\n", g_get_prgname ());
 }
 
 void
@@ -157,30 +134,52 @@ debug_printf (int level, const char *format, ...)
 static gboolean
 apache_cmd (const char *cmd)
 {
-	const char *argv[8];
-	char *cwd, *conf;
+	GPtrArray *argv;
+	char *server_root, *cwd, *pid_file;
+#ifdef HAVE_APACHE_2_4
+	char *default_runtime_dir;
+#endif
 	int status;
 	gboolean ok;
 
+	server_root = g_test_build_filename (G_TEST_BUILT, "", NULL);
+
 	cwd = g_get_current_dir ();
-	conf = g_build_filename (cwd, "httpd.conf", NULL);
+#ifdef HAVE_APACHE_2_4
+	default_runtime_dir = g_strdup_printf ("DefaultRuntimeDir %s", cwd);
+#endif
+	pid_file = g_strdup_printf ("PidFile %s/httpd.pid", cwd);
 
-	argv[0] = APACHE_HTTPD;
-	argv[1] = "-d";
-	argv[2] = cwd;
-	argv[3] = "-f";
-	argv[4] = conf;
-	argv[5] = "-k";
-	argv[6] = cmd;
-	argv[7] = NULL;
+	argv = g_ptr_array_new ();
+	g_ptr_array_add (argv, APACHE_HTTPD);
+	g_ptr_array_add (argv, "-d");
+	g_ptr_array_add (argv, server_root);
+	g_ptr_array_add (argv, "-f");
+	g_ptr_array_add (argv, "httpd.conf");
 
-	ok = g_spawn_sync (cwd, (char **)argv, NULL, 0, NULL, NULL,
+#ifdef HAVE_APACHE_2_4
+	g_ptr_array_add (argv, "-c");
+	g_ptr_array_add (argv, default_runtime_dir);
+#endif
+	g_ptr_array_add (argv, "-c");
+	g_ptr_array_add (argv, pid_file);
+
+	g_ptr_array_add (argv, "-k");
+	g_ptr_array_add (argv, (char *)cmd);
+	g_ptr_array_add (argv, NULL);
+
+	ok = g_spawn_sync (cwd, (char **)argv->pdata, NULL, 0, NULL, NULL,
 			   NULL, NULL, &status, NULL);
 	if (ok)
 		ok = (status == 0);
 
+	g_free (server_root);
 	g_free (cwd);
-	g_free (conf);
+	g_free (pid_file);
+#ifdef HAVE_APACHE_2_4
+	g_free (default_runtime_dir);
+#endif
+	g_ptr_array_free (argv, TRUE);
 
 	return ok;
 }
@@ -228,15 +227,18 @@ soup_test_session_new (GType type, ...)
 	va_list args;
 	const char *propname;
 	SoupSession *session;
+	char *cafile;
 
 	va_start (args, type);
 	propname = va_arg (args, const char *);
 	session = (SoupSession *)g_object_new_valist (type, propname, args);
 	va_end (args);
 
+	cafile = g_test_build_filename (G_TEST_DIST, "test-cert.pem", NULL);
 	g_object_set (G_OBJECT (session),
-		      SOUP_SESSION_SSL_CA_FILE, SRCDIR "/test-cert.pem",
+		      SOUP_SESSION_SSL_CA_FILE, cafile,
 		      NULL);
+	g_free (cafile);
 
 	if (http_debug_level && !logger) {
 		SoupLoggerLogLevel level = MIN ((SoupLoggerLogLevel)http_debug_level, SOUP_LOGGER_LOG_BODY);
@@ -253,16 +255,10 @@ soup_test_session_new (GType type, ...)
 void
 soup_test_session_abort_unref (SoupSession *session)
 {
-	g_object_add_weak_pointer (G_OBJECT (session), (gpointer *)&session);
-
 	soup_session_abort (session);
-	g_object_unref (session);
 
-	if (session) {
-		errors++;
-		debug_printf (1, "leaked SoupSession!\n");
-		g_object_remove_weak_pointer (G_OBJECT (session), (gpointer *)&session);
-	}
+	g_assert_cmpint (G_OBJECT (session)->ref_count, ==, 1);
+	g_object_unref (session);
 }
 
 static gpointer run_server_thread (gpointer user_data);
@@ -272,14 +268,14 @@ test_server_new (gboolean in_own_thread, gboolean ssl)
 {
 	SoupServer *server;
 	GMainContext *async_context;
-	const char *ssl_cert_file, *ssl_key_file;
+	char *ssl_cert_file, *ssl_key_file;
 	SoupAddress *addr;
 
 	async_context = in_own_thread ? g_main_context_new () : NULL;
 
 	if (ssl) {
-		ssl_cert_file = SRCDIR "/test-cert.pem";
-		ssl_key_file = SRCDIR "/test-key.pem";
+		ssl_cert_file = g_test_build_filename (G_TEST_DIST, "test-cert.pem", NULL);
+		ssl_key_file = g_test_build_filename (G_TEST_DIST, "test-key.pem", NULL);
 	} else
 		ssl_cert_file = ssl_key_file = NULL;
 
@@ -294,6 +290,8 @@ test_server_new (gboolean in_own_thread, gboolean ssl)
 	g_object_unref (addr);
 	if (async_context)
 		g_main_context_unref (async_context);
+	g_free (ssl_cert_file);
+	g_free (ssl_key_file);
 
 	if (!server) {
 		g_printerr ("Unable to create server\n");
@@ -344,9 +342,6 @@ soup_test_server_quit_unref (SoupServer *server)
 {
 	GThread *thread;
 
-	g_object_add_weak_pointer (G_OBJECT (server),
-				   (gpointer *)&server);
-
 	thread = g_object_get_data (G_OBJECT (server), "thread");
 	if (thread) {
 		soup_add_completion (soup_server_get_async_context (server),
@@ -354,14 +349,9 @@ soup_test_server_quit_unref (SoupServer *server)
 		g_thread_join (thread);
 	} else
 		soup_server_quit (server);
-	g_object_unref (server);
 
-	if (server) {
-		errors++;
-		debug_printf (1, "leaked SoupServer!\n");
-		g_object_remove_weak_pointer (G_OBJECT (server),
-					      (gpointer *)&server);
-	}
+	g_assert_cmpint (G_OBJECT (server)->ref_count, ==, 1);
+	g_object_unref (server);
 }
 
 typedef struct {
@@ -550,3 +540,93 @@ soup_test_request_close_stream (SoupRequest   *req,
 
 	return ok;
 }
+
+void
+soup_test_register_resources (void)
+{
+	static gboolean registered = FALSE;
+	GResource *resource;
+	char *path;
+	GError *error = NULL;
+
+	if (registered)
+		return;
+
+	path = g_test_build_filename (G_TEST_BUILT, "soup-tests.gresource", NULL);
+	resource = g_resource_load (path, &error);
+	if (!resource) {
+		g_printerr ("Could not load resource soup-tests.gresource: %s\n",
+			    error->message);
+		exit (1);
+	}
+	g_free (path);
+
+	g_resources_register (resource);
+	g_resource_unref (resource);
+
+	registered = TRUE;
+}
+
+SoupBuffer *
+soup_test_load_resource (const char  *name,
+			 GError     **error)
+{
+	GBytes *bytes;
+	char *path;
+
+	soup_test_register_resources ();
+
+	path = g_build_path ("/", "/org/gnome/libsoup/tests/resources", name, NULL);
+	bytes = g_resources_lookup_data (path, G_RESOURCE_LOOKUP_FLAGS_NONE, error);
+	g_free (path);
+
+	if (!bytes)
+		return NULL;
+
+	return soup_buffer_new_with_owner (g_bytes_get_data (bytes, NULL),
+					   g_bytes_get_size (bytes),
+					   bytes,
+					   (GDestroyNotify) g_bytes_unref);
+}
+
+SoupBuffer *
+soup_test_get_index (void)
+{
+	if (!index_buffer) {
+		char *path, *contents;
+		gsize length;
+		GError *error = NULL;
+
+		path = g_test_build_filename (G_TEST_DIST, "index.txt", NULL);
+		if (!g_file_get_contents (path, &contents, &length, &error)) {
+			g_printerr ("Could not read index.txt: %s\n",
+				    error->message);
+			exit (1);
+		}
+		g_free (path);
+
+		index_buffer = soup_buffer_new (SOUP_MEMORY_TAKE, contents, length);
+	}
+
+	return index_buffer;
+}
+
+#ifndef G_HAVE_ISO_VARARGS
+void
+soup_test_assert (gboolean expr, const char *fmt, ...)
+{
+	char *message;
+	va_list args;
+
+	if (G_UNLIKELY (!expr)) {
+		va_start (args, fmt);
+		message = g_strdup_vprintf (fmt, args);
+		va_end (args);
+
+		g_assertion_message (G_LOG_DOMAIN,
+				     "???", 0, "???"
+				     message);
+		g_free (message);
+	}
+}
+#endif
